@@ -7,6 +7,7 @@ or multiexonic and with at least one intron in common.
 """
 
 import itertools
+import importlib
 import logging
 from ..transcripts.transcript import Transcript
 from .abstractlocus import Abstractlocus
@@ -82,7 +83,6 @@ class Sublocus(Abstractlocus):
 
         # This part is necessary to import modules
         if hasattr(self.configuration, "modules"):
-            import importlib
             for mod in self.configuration.modules:
                 globals()[mod] = importlib.import_module(mod)
 
@@ -138,14 +138,14 @@ class Sublocus(Abstractlocus):
         state["excluded"] = self.excluded.as_dict()
         return state
 
-    def load_dict(self, state):
-        super().load_dict(state)
+    def load_dict(self, state, load_transcripts=True, load_configuration=True):
+        super().load_dict(state, load_transcripts=load_transcripts, load_configuration=load_configuration)
         self.monosubloci = []
         for stat in state["monosubloci"]:
             s = Monosublocus()
             s.load_dict(stat)
             self.monosubloci.append(s)
-        self.excluded = Excluded()
+        self.excluded = Excluded(configuration=self.configuration)
         self.excluded.load_dict(state["excluded"])
 
     def add_transcript_to_locus(self, transcript: Transcript, check_in_locus=True, **kwargs):
