@@ -9,11 +9,11 @@ and is used to define all the possible children (subloci, monoloci, loci, etc.)
 # Core imports
 import collections
 import networkx
+from Mikado.serializers.blast_serializer import Hsp
 from sqlalchemy import bindparam
 from sqlalchemy.engine import Engine
 from sqlalchemy.ext import baked
-from sqlalchemy.orm.session import sessionmaker
-from sqlalchemy.orm import session as sasession
+from sqlalchemy.orm.session import sessionmaker, Session
 from sqlalchemy.sql.expression import and_
 from .abstractlocus import Abstractlocus
 from .monosublocusholder import MonosublocusHolder
@@ -21,8 +21,8 @@ from .sublocus import Sublocus
 from ..exceptions import NotInLocusError
 from ..parsers.GFF import GffLine
 from ..serializers.blast_serializer import Hit, Query, Target
-from ..serializers.external import External
-from ..serializers.junction import Junction, Chrom
+from ..serializers.external import External, ExternalSource
+from ..serializers.junction import Junction
 from ..serializers.orf import Orf
 from ..transcripts import Transcript
 from ..utilities import dbutils, grouper
@@ -33,15 +33,50 @@ import functools
 from collections import OrderedDict as SortedDict
 from .locus import Locus
 from .excluded import Excluded
-from typing import Union
+from typing import Union, List, Dict
 from ..utilities import Interval, IntervalTree
 from itertools import combinations
 import random
+import asyncio
 
 # The number of attributes is something I need
 # pylint: disable=too-many-instance-attributes
 
 
+bakery = baked.bakery()
+
+junction_baked = bakery(lambda session: session.query(Junction))
+junction_baked += lambda q: q.filter(and_(
+    Junction.chrom == bindparam("chrom"),
+    Junction.junction_start >= bindparam("junctionStart"),
+    Junction.junction_end <= bindparam("junctionEnd")
+))
+
+hit_baked = bakery(lambda session: session.query(Hit))
+hit_baked += lambda q: q.filter(and_(
+    Hit.query_id.in_(bindparam("queries", expanding=True)),
+    Hit.evalue <= bindparam("evalue"),
+    Hit.hit_number <= bindparam("hit_number")
+))
+
+hsp_baked = bakery(lambda session: session.query(Hsp))
+hsp_baked += lambda q: q.filter(and_(Hsp.hsp_evalue <= bindparam("hsp_evalue"),
+                                     Hsp.query_id.in_(bindparam("queries", expanding=True))))
+
+
+query_baked = bakery(lambda session: session.query(Query))
+query_baked += lambda q: q.filter(Query.query_name.in_(bindparam("tids", expanding=True)))
+
+target_baked = bakery(lambda session: session.query(Target))
+target_baked += lambda q: q.filter(Target.target_id.in_(bindparam("targets", expanding=True)))
+
+source_bakery = bakery(lambda session: session.query(ExternalSource))
+source_bakery += lambda q: q.filter(ExternalSource.source.in_(bindparam("sources", expanding=True)))
+
+orfs_baked = bakery(lambda session: session.query(Orf))
+orfs_baked += lambda q: q.filter(Orf.query_id.in_(bindparam("queries", expanding=True)))
+
+
 class Superlocus(Abstractlocus):
     """The superlocus class is used to define overlapping regions
     on the genome, and it receives as input transcript class instances.
@@ -49,24 +84,7 @@ class Superlocus(Abstractlocus):
 
     __name__ = "superlocus"
 
-    bakery = baked.bakery()
-    db_baked = bakery(lambda session: session.query(Chrom))
-    db_baked += lambda q: q.filter(Chrom.name == bindparam("chrom_name"))
 
-    junction_baked = bakery(lambda session: session.query(Junction))
-    junction_baked += lambda q: q.filter(and_(
-        Junction.chrom == bindparam("chrom"),
-        Junction.junction_start >= bindparam("junctionStart"),
-        Junction.junction_end <= bindparam("junctionEnd"),
-        Junction.strand == bindparam("strand")
-    ))
-
-    hit_baked = bakery(lambda session: session.query(Hit))
-    hit_baked += lambda q: q.filter(and_(
-        Hit.query_id == bindparam("query_id"),
-        Hit.evalue <= bindparam("evalue"),
-        Hit.hit_number <= bindparam("hit_number")
-    ))
 
     _complex_limit = (5000, 5000)
 
@@ -142,7 +160,6 @@ class Superlocus(Abstractlocus):
         self.engine = self.sessionmaker = self.session = None
         # Excluded object
         self.excluded = Excluded(configuration=self.configuration)
-        self.__retained_sources = set()
         self.__data_loaded = False
         self.__lost = dict()
         if transcript_instance is not None:
@@ -206,33 +223,6 @@ class Superlocus(Abstractlocus):
 
         return lines
 
-    def __create_monolocus_lines(self, superlocus_line: GffLine, new_id: str, print_cds=True):
-
-        """
-        Private method to prepare the lines for printing out monosubloci
-        into GFF/GTF files.
-        """
-
-        lines = []
-        self.define_monosubloci()
-        if len(self.monosubloci) > 0:
-            source = "{0}_monosubloci".format(self.source)
-            superlocus_line.source = source
-            lines.append(str(superlocus_line))
-            found = dict()
-            for monosublocus_instance in self.monosubloci:
-                monosublocus_instance.source = source
-                monosublocus_instance.parent = new_id
-                if monosublocus_instance.id in found:
-                    found[monosublocus_instance.id] += 1
-                    monosublocus_instance.counter = found[monosublocus_instance.id]
-                else:
-                    found[monosublocus_instance.id] = 0
-
-                lines.append(monosublocus_instance.__str__(print_cds=print_cds).rstrip())
-
-        return lines
-
     def __create_sublocus_lines(self, superlocus_line: GffLine, new_id: str, print_cds=True):
         """
         Private method to prepare the lines for printing out subloci
@@ -245,10 +235,8 @@ class Superlocus(Abstractlocus):
         self.define_subloci()
         found = dict()
         for sublocus_instance in self.subloci:
-            try:
-                sublocus_instance.source = source
-            except AttributeError:
-                raise AttributeError(sublocus_instance)
+            assert hasattr(sublocus_instance, "source"), sublocus_instance
+            sublocus_instance.source = source
             sublocus_instance.parent = new_id
             if sublocus_instance.id in found:
                 found[sublocus_instance.id] += 1
@@ -269,8 +257,7 @@ class Superlocus(Abstractlocus):
         :param level: level which we wish to print for. Can be "loci", "subloci", "monosubloci"
         :return: formatted GFF strings
         """
-        return self.__str__(print_cds=print_cds,
-                            level=level)
+        return self.__str__(print_cds=print_cds, level=level)
 
     def __str__(self, level=None, print_cds=True):
 
@@ -293,6 +280,8 @@ class Superlocus(Abstractlocus):
         if abs(self.start) == float("inf") or abs(self.start) == maxsize:
             return ''
 
+        assert level in (None, "loci", "subloci", "monosubloci"), f"Unrecognized level: {level}"
+
         superlocus_line = GffLine('')
         superlocus_line.chrom = self.chrom
         superlocus_line.feature = self.__name__
@@ -305,16 +294,10 @@ class Superlocus(Abstractlocus):
         superlocus_line.id, superlocus_line.name = new_id, self.name
         if self.approximation_level > 0:
             superlocus_line.attributes["approximation_level"] = self.approximation_level
-        if len(self.__retained_sources) > 0:
-            superlocus_line.attributes["retained_sources"] = ",".join(
-                sorted(list(self.__retained_sources))
-            )
 
         lines = []
-        if level not in (None, "loci", "subloci", "monosubloci"):
-            raise ValueError("Unrecognized level: {0}".format(level))
 
-        elif level == "loci" or (level is None and self.loci_defined is True):
+        if level == "loci" or (level is None and self.loci_defined is True):
             lines = self.__create_locus_lines(
                 superlocus_line,
                 new_id,
@@ -324,9 +307,6 @@ class Superlocus(Abstractlocus):
             lines = self.__create_monolocus_holder_lines(superlocus_line,
                                                          new_id,
                                                          print_cds=print_cds)
-            # lines = self.__create_monolocus_lines(superlocus_line,
-            #                                       new_id,
-            #                                       print_cds=print_cds)
         elif level == "subloci" or (level is None and self.monosubloci_defined is False):
             lines = self.__create_sublocus_lines(superlocus_line,
                                                  new_id,
@@ -369,33 +349,38 @@ class Superlocus(Abstractlocus):
         state["excluded"] = self.excluded.as_dict()
         return state
 
-    def load_dict(self, state, print_subloci=True, print_monoloci=True, load_transcripts=True):
+    def load_dict(self, state, print_subloci=True, print_monoloci=True, load_transcripts=True,
+                  load_configuration=True):
         """Method to reconstitute a Superlocus from a dumped dictionary."""
 
-        super().load_dict(state, load_transcripts=load_transcripts)
+        super().load_dict(state, load_transcripts=load_transcripts, load_configuration=load_configuration)
         self.loci = dict()
         for lid, stat in state["loci"].items():
             locus = Locus()
             locus.load_dict(stat)
             self.loci[lid] = locus
 
-        if print_subloci is True or print_monoloci is True:
+        if print_subloci is True:
+            self.excluded = Excluded(configuration=self.configuration)
+            self.excluded.load_dict(state["excluded"])
+            self.subloci = []
+            for stat in state["subloci"]:
+                sub = Sublocus(configuration=self.configuration)
+                sub.load_dict(stat)
+                assert isinstance(sub, Sublocus)
+                self.subloci.append(sub)
+        else:
+            self.subloci = []
+            self.excluded = Excluded(configuration=self.configuration)
 
-            if print_subloci is True:
-                self.excluded = Excluded()
-                self.excluded.load_dict(state["excluded"])
-                self.subloci = []
-                for stat in state["subloci"]:
-                    sub = Sublocus(configuration=self.configuration)
-                    sub.load_dict(stat)
-                    assert isinstance(sub, Sublocus)
-                    self.subloci.append(sub)
-            if print_monoloci is True:
-                self.monoholders = []
-                for stat in state["monoholders"]:
-                    sub = MonosublocusHolder()
-                    sub.load_dict(stat)
-                    self.monoholders.append(sub)
+        if print_monoloci is True:
+            self.monoholders = []
+            for stat in state["monoholders"]:
+                sub = MonosublocusHolder()
+                sub.load_dict(stat)
+                self.monoholders.append(sub)
+        else:
+            self.monoholders = []
         self.chrom, self.strand, self.start, self.end = state["chrom"], state["strand"], state["start"], state["end"]
         if len(self.loci) > 0 or len(self.transcripts) > 0:
             assert self.start != maxsize
@@ -462,16 +447,25 @@ class Superlocus(Abstractlocus):
             for new_locus in iter(sorted(new_loci)):
                 yield new_locus
 
-    def connect_to_db(self, engine: Engine):
+    def connect_to_db(self, engine: Engine, session: Session):
 
         """
         :param engine: the connection pool
         :type engine: Engine
 
+        :param session: a preformed session
+        :type session: Session
+
         This method will connect to the database using the information
         contained in the JSON configuration.
         """
 
+        if isinstance(session, Session):
+            self.session = session
+            self.sessionmaker = sessionmaker()
+            self.sessionmaker.configure(bind=self.session.bind)
+            self.engine = self.session.bind
+
         if engine is None:
             self.engine = dbutils.connect(self.configuration)
         else:
@@ -514,7 +508,7 @@ class Superlocus(Abstractlocus):
         del data_dict
         return to_remove, to_add
 
-    def _load_introns(self):
+    async def _load_introns(self):
 
         """Private method to load the intron data into the locus.
         :param data_dict: Dictionary containing the preloaded data, if available.
@@ -522,25 +516,18 @@ class Superlocus(Abstractlocus):
         """
 
         if len(self.introns) == 0:
-            if self.monoexonic is False:
-                raise ValueError("%s is multiexonic but has no introns defined!",
-                                 self.id)
+            assert self.monoexonic is True, f"{self.id} is multiexonic but has no introns defined!"
             self.logger.debug("No introns for %s", self.id)
             return
 
         self.logger.debug("Querying the DB for introns, %d total", len(self.introns))
         if not self.configuration.db_settings.db:
             return  # No data to load
-        # dbquery = self.db_baked(self.session).params(chrom_name=self.chrom).all()
 
-        ver_introns = self.engine.execute(" ".join([
-            "select junction_start, junction_end, strand from junctions where",
-            "chrom_id = (select chrom_id from chrom where name = \"{chrom}\")",
-            "and junction_start > {start} and junction_end < {end}"]).format(
-                chrom=self.chrom, start=self.start, end=self.end
-        ))
-        ver_introns = dict(((junc.junction_start, junc.junction_end), junc.strand)
-                           for junc in ver_introns)
+        ver_introns = collections.defaultdict(set)
+        for junc in junction_baked(self.session).params(chrom=self.chrom,
+                                                        junctionStart=self.start, junctionEnd=self.end):
+            ver_introns[(junc.junction_start, junc.junction_end)].add(junc.strand)
 
         self.logger.debug("Found %d verifiable introns for %s",
                           len(ver_introns), self.id)
@@ -549,13 +536,101 @@ class Superlocus(Abstractlocus):
             self.logger.debug("Checking %s%s:%d-%d",
                               self.chrom, self.strand, intron[0], intron[1])
             if (intron[0], intron[1]) in ver_introns:
-                self.logger.debug("Verified intron %s:%d-%d",
-                                  self.chrom, intron[0], intron[1])
-                self.locus_verified_introns.add((intron[0],
-                                                 intron[1],
-                                                 ver_introns[(intron[0], intron[1])]))
+                if self.stranded is False:
+                    for strand in ver_introns[(intron[0], intron[1])]:
+                        self.locus_verified_introns.add((intron[0],
+                                                         intron[1],
+                                                         strand))
+                elif self.strand in ver_introns[(intron[0], intron[1])]:
+                    self.locus_verified_introns.add((intron[0],
+                                                     intron[1],
+                                                     self.strand))
 
-    def _create_data_dict(self, engine, tid_keys) -> dict:
+    async def get_sources(self):
+        if self.configuration.pick.output_format.report_all_external_metrics is True:
+            sources = dict((source.source_id, source) for source in self.session.query(ExternalSource))
+        else:
+            sources = set()
+            sources.update({param for param in self.configuration.scoring.requirements.parameters.keys()
+                            if param.startswith("external")})
+            sources.update({param for param in self.configuration.scoring.not_fragmentary.parameters.keys()
+                            if param.startswith("external")})
+            sources.update({param for param in self.configuration.scoring.cds_requirements.parameters.keys()
+                            if param.startswith("external")})
+            sources.update({param for param in self.configuration.scoring.as_requirements.parameters.keys()
+                            if param.startswith("external")})
+            sources.update({param for param in self.configuration.scoring.scoring.keys()
+                            if param.startswith("external")})
+            sources = {param.replace("external.", "") for param in sources}
+            sources = dict((source.source_id, source) for source in
+                           source_bakery(self.session).params(sources=list(sources)))
+        return sources
+
+    async def get_external(self, query_ids, qids):
+        external = collections.defaultdict(dict)
+        sources = await self.get_sources()
+        baked = External.__table__.select().where(and_(External.source_id.in_(list(sources.keys())),
+                                                       External.query_id.in_(qids)))
+        for ext in self.session.execute(baked):
+            source_id, query_id, score = ext.source_id, ext.query_id, ext.score
+            assert source_id in sources and query_id in qids
+            rtype = sources[source_id].rtype
+            assert rtype in ("int", "float", "bool"), f"Invalid rtype: {rtype}"
+            if rtype == "int":
+                score = int(score)
+            elif rtype == "float":
+                score = float(score)
+            elif rtype == "bool":
+                score = bool(int(score))
+            external[query_ids[ext.query_id].query_name][
+                sources[ext.source_id].source] = (score, sources[ext.source_id].valid_raw)
+        return external
+
+    async def get_hits(self, query_ids, qids):
+        hsps = dict()
+        targets = set()
+        hits = collections.defaultdict(list)
+        for hsp in hsp_baked(self.session).params(
+                hsp_evalue=self.configuration.pick.chimera_split.blast_params.hsp_evalue,
+                queries=qids):
+            if hsp.query_id not in hsps:
+                hsps[hsp.query_id] = collections.defaultdict(list)
+            hsps[hsp.query_id][hsp.target_id].append(hsp)
+            targets.add(hsp.target_id)
+
+        target_ids = dict((target.target_id, target) for target in
+                          target_baked(self.session).params(targets=list(targets)))
+        current_hit = None
+        for hit in hit_baked(self.session).params(
+                evalue=self.configuration.pick.chimera_split.blast_params.evalue,
+                hit_number=self.configuration.pick.chimera_split.blast_params.max_target_seqs,
+                queries=qids):
+            if current_hit != hit.query_id:
+                current_hit = hit.query_id
+            current_counter = 0
+
+            current_counter += 1
+
+            my_query = query_ids[hit.query_id]
+            my_target = target_ids[hit.target_id]
+
+            hits[my_query.query_name].append(
+                Hit.as_full_dict_static(
+                    hit,
+                    hsps[hit.query_id][hit.target_id],
+                    my_query,
+                    my_target
+                )
+            )
+        return hits
+
+    async def get_orfs(self, qids) -> Dict[str, List]:
+        orfs = collections.defaultdict(list)
+        for orf in orfs_baked(self.session).params(queries=qids):
+            orfs[orf.query].append(orf.as_bed12())
+        return orfs
+
+    async def _create_data_dict(self, engine, tid_keys) -> dict:
 
         """Private method to retrieve data from the database and prepare it to be passed to the transcript
         instances.
@@ -571,9 +646,10 @@ class Superlocus(Abstractlocus):
         data_dict = dict()
         self.logger.debug("Starting to load hits and orfs for %d transcripts",
                           len(tid_keys))
-        data_dict["hits"] = collections.defaultdict(list)
-        data_dict["orfs"] = collections.defaultdict(list)
+
         data_dict["external"] = collections.defaultdict(dict)
+        data_dict["orfs"] = collections.defaultdict(list)
+        data_dict["hits"] = collections.defaultdict(list)
         if engine is None:
             return data_dict
 
@@ -581,92 +657,13 @@ class Superlocus(Abstractlocus):
             query_ids = dict((query.query_id, query) for query in
                              self.session.query(Query).filter(
                                  Query.query_name.in_(tid_group)))
-            # Retrieve the external scores
-
-            if query_ids:
-                external = self.session.query(External).filter(External.query_id.in_(query_ids.keys()))
-            else:
-                external = []
-
-            for ext in external:
-                if ext.rtype == "int":
-                    score = int(ext.score)
-                elif ext.rtype == "float":
-                    score = float(ext.score)
-                elif ext.rtype == "bool":
-                    score = bool(int(ext.score))
-                else:
-                    raise ValueError("Invalid rtype: {}".format(ext.rtype))
-
-                data_dict["external"][ext.query][ext.source] = (score, ext.valid_raw)
-
-            # Load the ORFs from the table
-            if query_ids:
-                orfs = self.session.query(Orf).filter(Orf.query_id.in_(query_ids.keys()))
-            else:
-                orfs = []
-
-            for orf in orfs:
-                data_dict["orfs"][orf.query].append(orf.as_bed12())
-
-            # Now retrieve the HSPs from the BLAST HSP table
-            hsp_command = " ".join([
-                "select * from hsp where",
-                "hsp_evalue <= {0} and query_id in {1} order by query_id;"]).format(
-                self.configuration.pick.chimera_split.blast_params.hsp_evalue,
-                "({0})".format(", ".join([str(_) for _ in query_ids.keys()]))
-            )
-
-            hsps = dict()
-            targets = set()
-
-            for hsp in engine.execute(hsp_command):
-                if hsp.query_id not in hsps:
-                    hsps[hsp.query_id] = collections.defaultdict(list)
-                hsps[hsp.query_id][hsp.target_id].append(hsp)
-                targets.add(hsp.target_id)
-
-            # Now that we have the HSPs, load the corresponding HITs
-            hit_command = " ".join([
-                "select * from hit where evalue <= {0}",
-                "and hit_number <= {1} and query_id in {2}",
-                "order by query_id, evalue asc;"
-            ]).format(
-                self.configuration.pick.chimera_split.blast_params.evalue,
-                self.configuration.pick.chimera_split.blast_params.max_target_seqs,
-                "({0})".format(", ".join([str(_) for _ in query_ids.keys()])))
-
-            if len(targets) > 0:
-                target_ids = dict()
-                for target_group in grouper(targets, 100):
-                    target_ids.update(dict((target.target_id, target) for target in
-                                      self.session.query(Target).filter(
-                                          Target.target_id.in_(target_group))))
-            else:
-                target_ids = dict()
-            current_hit = None
-            for hit in engine.execute(hit_command):
-                if current_hit != hit.query_id:
-                    current_hit = hit.query_id
-                current_counter = 0
-
-                current_counter += 1
-
-                my_query = query_ids[hit.query_id]
-                my_target = target_ids[hit.target_id]
-
-                data_dict["hits"][my_query.query_name].append(
-                    Hit.as_full_dict_static(
-                        hit,
-                        hsps[hit.query_id][hit.target_id],
-                        my_query,
-                        my_target
-                    )
-                )
-
+            qids = list(query_ids.keys())
+            data_dict["orfs"].update(await self.get_orfs(qids))
+            data_dict["external"].update(await self.get_external(query_ids, qids))
+            data_dict["hits"].update(await self.get_hits(query_ids=query_ids, qids=qids))
         return data_dict
 
-    def load_all_transcript_data(self, engine=None):
+    def load_all_transcript_data(self, engine=None, session=None):
 
         """
         This method will load data into the transcripts instances,
@@ -686,14 +683,15 @@ class Superlocus(Abstractlocus):
             self.approximation_level = 1
             self.reduce_method_one(None)
 
-        self.connect_to_db(engine)
+        self.connect_to_db(engine, session)
 
         tid_keys = list(self.transcripts.keys())
         to_remove, to_add = set(), dict()
         # This will function even if data_dict is None
-        self._load_introns()
-
+        intron_loader = self._load_introns()
         data_dict = self._create_data_dict(engine, tid_keys)
+        data_dict = asyncio.run(data_dict)
+        asyncio.run(intron_loader)
 
         self.logger.debug("Verified %d introns for %s",
                           len(self.locus_verified_introns),
@@ -701,8 +699,8 @@ class Superlocus(Abstractlocus):
 
         self.logger.debug("Finished retrieving data for %d transcripts",
                           len(tid_keys))
-        self.session.close()
-        sasession.close_all_sessions()
+        # self.session.close()
+        # sasession.close_all_sessions()
 
         for tid in tid_keys:
             remove_flag, new_transcripts = self.load_transcript_data(tid, data_dict)
@@ -1002,33 +1000,7 @@ class Superlocus(Abstractlocus):
                                     use_transcript_scores=self._use_transcript_scores
                                     )
             new_sublocus.logger = self.logger
-            for ttt in subl[1:]:
-                try:
-                    new_sublocus.add_transcript_to_locus(ttt)
-                except NotInLocusError as orig_exc:
-                    exc_text = """Sublocus: {0}
-                    Offending transcript:{1}
-                    In locus manual check: {2}
-                    Original exception: {3}""".format(
-                        "{0} {1}:{2}-{3} {4}".format(
-                            subl[0].id, subl[0].chrom, subl[0].start,
-                            subl[0].end, subl[0].exons),
-                        "{0} {1}:{2}-{3} {4}".format(ttt.id, ttt.chrom,
-                                                     ttt.start, ttt.end, ttt.exons),
-                        "Chrom {0} Strand {1} overlap {2}".format(
-                            new_sublocus.chrom == ttt.chrom,
-                            "{0}/{1}/{2}".format(
-                                new_sublocus.strand,
-                                ttt.strand,
-                                new_sublocus.strand == ttt.strand
-                            ),
-                            self.overlap((subl[0].start, subl[1].end),
-                                         (ttt.start, ttt.end)) > 0
-                        ),
-                        orig_exc
-                    )
-                    raise NotInLocusError(exc_text)
-
+            [new_sublocus.add_transcript_to_locus(ttt) for ttt in subl[1:]]
             new_sublocus.parent = self.id
             new_sublocus.metrics_calculated = False
             new_sublocus.get_metrics()
@@ -1178,7 +1150,7 @@ class Superlocus(Abstractlocus):
 
         loci = []
         for monoholder in self.monoholders:
-            monoholder.define_loci(purge=self.purge, check_requirements=check_requirements)
+            monoholder.define_loci(check_requirements=check_requirements)
             for locus_instance in monoholder.loci:
                 monoholder.loci[locus_instance].parent = self.id
                 loci.append(monoholder.loci[locus_instance])
@@ -1195,8 +1167,8 @@ class Superlocus(Abstractlocus):
         if self.configuration.pick.alternative_splicing.report is True:
             self.define_alternative_splicing()
 
-        self.__find_lost_transcripts()
-        while len(self.lost_transcripts) > 0:
+        self._find_lost_transcripts()
+        while len(self.lost_transcripts):
             new_locus = None
             for transcript in self.lost_transcripts.values():
                 if new_locus is None:
@@ -1226,13 +1198,16 @@ class Superlocus(Abstractlocus):
 
         return
 
-    def __find_lost_transcripts(self):
+    def _find_lost_transcripts(self):
 
         """Private method to identify, after defining loci, all transcripts that are not intersecting any
         of the resulting genes and that therefore could constitute "lost" genes.
         This could happen if e.g. a valid transcript is deselected in the first stage ("sublocus") as it intersects a
         longer transcript, which will itself be subsequently deselected in comparison with another shorter one. This
         would leave the first transcript stranded.
+        WARNING: if a transcript's score has fallen at or below 0, and self.purge is False, transcripts will *not*
+        be recovered by this method. Mikado will presume that excluding them was the correct thing to do and ignore
+        them henceforth.
         """
 
         self.__lost = dict()
@@ -1271,7 +1246,8 @@ class Superlocus(Abstractlocus):
                     to_remove.add(tid)
             not_loci_transcripts = set.difference(not_loci_transcripts, to_remove)
 
-        self.__lost.update({tid: self.transcripts[tid] for tid in not_loci_transcripts})
+        self.__lost.update({tid: self.transcripts[tid] for tid in not_loci_transcripts
+                            if (self.purge is False or self.transcripts[tid].score > 0)})
 
         if len(self.__lost):
             self.logger.debug("Lost %s transcripts from %s; starting the recovery process: TIDs: %s",
@@ -1310,15 +1286,8 @@ class Superlocus(Abstractlocus):
 
         loci_cliques = dict()
         for lid, locus_instance in self.loci.items():
-            try:
-                neighbors = set(t_graph.neighbors(locus_instance.primary_transcript_id))
-            except networkx.exception.NetworkXError:
-                raise networkx.exception.NetworkXError(
-                    "{} {}".format(
-                    # locus_instance.primary_transcript.attributes["Alias"],
-                    locus_instance.primary_transcript_id,
-                    list(t_graph.nodes)
-                ))
+            assert locus_instance.primary_transcript_id in t_graph.nodes()
+            neighbors = set(t_graph.neighbors(locus_instance.primary_transcript_id))
             loci_cliques[lid] = neighbors
             self.logger.debug("Neighbours for %s: %s", lid, neighbors)
 
@@ -1447,7 +1416,7 @@ class Superlocus(Abstractlocus):
 
         # Now the monoexonic
         monoexonic = IntervalTree()
-        [monoexonic.add_interval(interval) for interval in monos]
+        [monoexonic.add(interval) for interval in monos]
         monos = sorted(monos)
         for mono in monos:
             edges.update(set((mono.value, omono.value) for omono in
@@ -1493,7 +1462,7 @@ class Superlocus(Abstractlocus):
                 start, end = sorted([transcript.selected_cds_start, transcript.selected_cds_end])
             else:
                 start, end = transcript.start, transcript.end
-            itree.add_interval(Interval(start, end, transcript.id))
+            itree.add(Interval(start, end, transcript.id))
 
         for tid in self.transcripts:
             if tid in primaries:
@@ -1537,23 +1506,22 @@ class Superlocus(Abstractlocus):
             return False  # We do not want intersection with oneself
 
         if transcript.monoexonic is False and other.monoexonic is False:
-            if cds_only is False or transcript.is_coding is False or other.is_coding is False:
-                intersection = set.intersection(transcript.introns, other.introns)
-            else:
+            if all([cds_only, transcript.is_coding, other.is_coding]):
                 intersection = set.intersection(transcript.selected_cds_introns,
                                                 other.selected_cds_introns)
-            intersecting = (len(intersection) > 0)
-
-        elif transcript.monoexonic is True and other.monoexonic is True:
-
-            if cds_only is False or transcript.is_coding is False or other.is_coding is False:
-                intersecting = (cls.overlap(
-                    (transcript.start, transcript.end),
-                    (other.start, other.end), positive=False) > 0)
             else:
+                intersection = set.intersection(transcript.introns, other.introns)
+
+            intersecting = (len(intersection) > 0)
+        elif transcript.monoexonic is True and other.monoexonic is True:
+            if all([cds_only, transcript.is_coding, other.is_coding]):
                 intersecting = (cls.overlap(
                     (transcript.selected_cds_start, transcript.selected_cds_end),
                     (other.selected_cds_start, other.selected_cds_end), positive=False) > 0)
+            else:
+                intersecting = (cls.overlap(
+                    (transcript.start, transcript.end),
+                    (other.start, other.end), positive=False) > 0)
         else:
             intersecting = False
 
