@@ -13,15 +13,20 @@ import os
 import sys
 import logging
 import logging.handlers
+from typing import Union
+
 from ..utilities import path_join, comma_split
-from ..utilities.log_utils import create_default_logger, formatter
+from ..utilities.log_utils import create_default_logger, create_null_logger
+from ._utils import check_log_settings_and_create_logger
 from ..utilities import dbutils
-from ..exceptions import InvalidJson
+from ..exceptions import InvalidConfiguration
 from ..exceptions import InvalidSerialization
 import random
 from ..utilities import blast_keys
 from ..configuration import MikadoConfiguration, DaijinConfiguration
 import pysam
+import sqlalchemy
+from ..configuration import configurator
 
 
 __author__ = 'Luca Venturini'
@@ -53,11 +58,11 @@ def xml_launcher(xml_candidate=None, configuration=None, logger=None):
     xml_serializer()
 
 
-def load_junctions(args, logger):
+def load_junctions(mikado_configuration: Union[MikadoConfiguration, DaijinConfiguration], logger):
     """
     Function that performs the loading of the junctions.
 
-    :param args: the Namespace with all the details from the command line.
+    :param mikado_configuration: Mikado configuration object
 
     :param logger: the logging instance.
     :type logger: (None | logging.Logger)
@@ -65,54 +70,54 @@ def load_junctions(args, logger):
     :return:
     """
 
-    assert isinstance(args.configuration, (MikadoConfiguration, DaijinConfiguration))
+    assert isinstance(mikado_configuration, (MikadoConfiguration, DaijinConfiguration))
 
-    if not args.configuration.serialise.files.junctions:
+    if not mikado_configuration.serialise.files.junctions:
         logger.info("Skipping junction loading as no junctions have been provided.")
         return
 
-    if not args.configuration.reference.genome:
-        exc = InvalidJson(
+    if not mikado_configuration.reference.genome:
+        exc = InvalidConfiguration(
             "Missing the genome FAI file for serialising the junctions. \
 I cannot proceed with this step!")
         logger.exception(exc)
         raise exc
 
     logger.info("Starting to load junctions: %s",
-                args.configuration.serialise.files.junctions)
+                mikado_configuration.serialise.files.junctions)
     for junction_file in iter(
-            j_file for j_file in args.configuration.serialise.files.junctions
+            j_file for j_file in mikado_configuration.serialise.files.junctions
             if j_file != ''):
         logger.debug("Loading junctions: %s", junction_file)
         from ..serializers import junction
         serializer = junction.JunctionSerializer(
             junction_file,
-            configuration=args.configuration,
+            configuration=mikado_configuration,
             logger=logger)
         serializer()
     logger.info("Loaded junctions")
 
 
-def load_blast(args, logger):
+def load_blast(mikado_configuration: Union[MikadoConfiguration, DaijinConfiguration], logger):
 
     """
     Function to load the BLAST data into the chosen database.
 
-    :param args: the Namespace with all the details from the command line.
+    :param mikado_configuration: Mikado configuration object
 
     :param logger: the logging instance.
     :type logger: (None | logging.Logger)
 
     """
-    if args.configuration.serialise.files.xml:
+    if mikado_configuration.serialise.files.xml:
         logger.info("Starting to load BLAST data")
         filenames = []
 
         part_launcher = functools.partial(
             xml_launcher,
-            **{"configuration": args.configuration, "logger": logger})
+            **{"configuration": mikado_configuration, "logger": logger})
 
-        for xml in args.configuration.serialise.files.xml:
+        for xml in mikado_configuration.serialise.files.xml:
             if os.path.isdir(xml):
                 filenames.extend(
                     [os.path.join(xml, _xml) for _xml in
@@ -128,52 +133,137 @@ def load_blast(args, logger):
             logger.warning("No valid BLAST file specified, skipping this phase")
 
 
-def load_orfs(args, logger):
+def load_orfs(mikado_configuration: Union[DaijinConfiguration, MikadoConfiguration], logger):
 
     """
     Function to load the ORFs into the DB.
-    :param args:
+    :param mikado_configuration: configuration object
     :param logger:
     :return:
     """
 
-    if len(args.configuration.serialise.files.orfs) > 0:
+    if len(mikado_configuration.serialise.files.orfs) > 0:
         from ..serializers import orf
         logger.info("Starting to load ORF data")
-        for orf_file in args.configuration.serialise.files.orfs:
+        for orf_file in mikado_configuration.serialise.files.orfs:
             logger.debug("Starting to load ORFs from %s", orf_file)
             try:
                 serializer = orf.OrfSerializer(orf_file,
-                                               configuration=args.configuration,
+                                               configuration=mikado_configuration,
                                                logger=logger)
                 serializer()
             except InvalidSerialization:
                 logger.critical("Mikado serialise failed due to problems with the input data. Please check the logs.")
-                os.remove(args.configuration.db_settings.db)
+                os.remove(mikado_configuration.db_settings.db)
                 sys.exit(1)
         logger.info("Finished loading ORF data")
     else:
         logger.info("No ORF data provided, skipping")
 
 
-def load_external(args, logger):
+def load_external(mikado_configuration: Union[MikadoConfiguration, DaijinConfiguration], logger):
 
     """Function to load external data from."""
 
-    if args.configuration.serialise.files.external_scores in (None, ""):
+    if mikado_configuration.serialise.files.external_scores in (None, ""):
         logger.debug("No external scores to load, returning")
         return
     else:
         logger.info("Starting to load external data")
         from ..serializers import external
         with external.ExternalSerializer(
-                args.configuration.serialise.files.external_scores,
-                configuration=args.configuration,
+                mikado_configuration.serialise.files.external_scores,
+                configuration=mikado_configuration,
                 logger=logger) as serializer:
             serializer()
         logger.info("Finished loading external data")
 
 
+def _set_serialise_files_options(conf: Union[MikadoConfiguration, DaijinConfiguration],
+                                 args, logger=create_null_logger()) -> Union[MikadoConfiguration, DaijinConfiguration]:
+    conf.serialise.files.orfs = args.orfs.split(",") if args.orfs else conf.serialise.files.orfs
+    conf.serialise.files.xml = args.xml.split(",") if args.xml else conf.serialise.files.xml
+    conf.serialise.files.junctions = args.junctions.split(",") if args.junctions else conf.serialise.files.junctions
+    conf.serialise.files.transcripts = args.transcripts if args.transcripts else conf.serialise.files.transcripts
+    conf.serialise.files.blast_targets = args.blast_targets if args.blast_targets else \
+        conf.serialise.files.blast_targets
+    conf.reference.genome_fai = args.genome_fai if args.genome_fai else \
+        conf.reference.genome_fai
+    conf.reference.genome = args.genome if args.genome else conf.reference.genome
+
+    conf.serialise.files.output_dir = args.output_dir if args.output_dir is not None else \
+        conf.serialise.files.output_dir
+
+    try:
+        os.makedirs(conf.serialise.files.output_dir, exist_ok=True)
+    except (OSError, PermissionError, FileExistsError) as exc:
+        logger.error("Failed to create the output directory!")
+        logger.exception(exc)
+        raise exc
+        
+    if args.db is not None:
+        conf.db_settings.db = args.db
+        conf.db_settings.dbtype = "sqlite"    
+
+    if conf.serialise.files.output_dir != "." and conf.db_settings.dbtype == "sqlite":
+        conf.db_settings.db = path_join(conf.serialise.files.output_dir,
+                                        os.path.basename(conf.db_settings.db))
+
+    if conf.serialise.files.junctions:
+        if conf.reference.genome_fai in (None, ""):
+            if conf.reference.genome not in (None, ""):
+                _ = pysam.Fastafile(conf.reference.genome)
+                conf.reference.genome_fai = conf.reference.genome + ".fai"
+            else:
+                exc = InvalidConfiguration("Missing FAI file for junction loading!")
+                logger.critical(exc)
+                raise exc
+
+    return conf
+
+
+def _set_serialise_run_options(conf: Union[MikadoConfiguration, DaijinConfiguration],
+                               args, logger=create_null_logger()) -> Union[MikadoConfiguration, DaijinConfiguration]:
+    conf.serialise.max_regression = args.max_regression or conf.serialise.max_regression
+    conf.serialise.start_adjustment = args.start_adjustment
+    conf.serialise.max_target_seqs = args.max_target_seqs or conf.serialise.max_target_seqs
+    conf.serialise.single_thread = args.single_thread or conf.serialise.single_thread
+    conf.serialise.files.blast_loading_debug = True if args.blast_loading_debug else \
+        conf.serialise.files.blast_loading_debug
+    conf.serialise.force = args.force if args.force is not None else conf.serialise.force
+    # File with the external scores
+    conf.serialise.files.external_scores = args.external_scores if args.external_scores else \
+        conf.serialise.files.external_scores
+    conf.serialise.codon_table = args.codon_table if isinstance(args.codon_table, (int, str)) else \
+        conf.serialise.codon_table
+    return conf
+
+
+def _execute_force_removal(configuration: Union[MikadoConfiguration, DaijinConfiguration],
+                           logger=create_null_logger()):
+    if (configuration.db_settings.dbtype == "sqlite" and
+            os.path.exists(configuration.db_settings.db)):
+        logger.warn("Removing old data from %s because force option in place",
+                    configuration.db_settings.db)
+        os.remove(configuration.db_settings.db)
+
+    engine = dbutils.connect(configuration)
+    meta = sqlalchemy.MetaData(bind=engine)
+    meta.reflect(engine)
+    for tab in reversed(meta.sorted_tables):
+        logger.debug("Dropping %s", tab)
+        tab.drop()
+        if configuration.db_settings.dbtype == "mysql":
+            engine.execute("OPTIMIZE TABLE {}".format(tab.name))
+    if configuration.db_settings.dbtype == "mysql":
+        engine.execute("")
+    # This would fail in MySQL as it uses the OPTIMIZE TABLE syntax above
+    elif configuration.db_settings.dbtype != "sqlite":
+        engine.execute("VACUUM")
+    dbutils.DBBASE.metadata.create_all(engine)
+    return
+
+
 def setup(args):
 
     """
@@ -182,182 +272,55 @@ def setup(args):
     :return:
     """
 
-    import sqlalchemy
-    from ..configuration import configurator
-    args.configuration = configurator.load_and_validate_config(args.configuration)
+    mikado_configuration = configurator.load_and_validate_config(args.configuration)
     logger = create_default_logger("serialiser")
-    # Get the log level from general settings
-    if args.start_method is not None:
-        args.configuration.multiprocessing_method = args.start_method
-
-    if args.procs is not None and args.procs > 0:
-        args.configuration.threads = args.procs
+    mikado_configuration.multiprocessing_method = args.start_method if args.start_method is not None else \
+        mikado_configuration.multiprocessing_method
+    mikado_configuration.threads = args.procs if (args.procs is not None and args.procs > 0) else \
+        mikado_configuration.threads
 
     # Retrieve data from the argparse and put it into the configuration
-    if args.orfs:
-        args.configuration.serialise.files.orfs = args.orfs.split(",")
-    if args.xml:
-        args.configuration.serialise.files.xml = args.xml.split(",")
-    if args.junctions:
-        args.configuration.serialise.files.junctions = args.junctions.split(",")
-    if args.transcripts is not None:
-        args.configuration.serialise.files.transcripts = args.transcripts
-    if args.blast_targets is not None and args.blast_targets:
-        args.configuration.serialise.files.blast_targets = args.blast_targets
-    if args.genome_fai is not None:
-        args.configuration.reference.genome_fai = args.genome_fai
-    if args.db is not None:
-        args.configuration.db_settings.db = args.db
-        args.configuration.db_settings.dbtype = "sqlite"
-    if args.output_dir is not None:
-        args.configuration.serialise.files.output_dir = args.output_dir
-
-    if args.configuration.serialise.files.output_dir != ".":
-        if args.configuration.db_settings.dbtype == "sqlite":
-            args.configuration.db_settings.db = os.path.basename(
-                args.configuration.db_settings.db)
-
-    if args.log_level is not None:
-        args.configuration.log_settings.log_level = args.log_level
-
-    args.configuration.serialise.force = args.force
-    args.configuration.serialise.max_regression = args.max_regression or args.configuration.serialise.max_regression
-    args.configuration.serialise.start_adjustment = args.start_adjustment
-    args.configuration.serialise.max_target_seqs = args.max_target_seqs or args.configuration.serialise.max_target_seqs
-    args.configuration.threads = args.procs or args.configuration.threads
-    args.configuration.serialise.single_thread = args.single_thread or args.configuration.serialise.single_thread
-
-    if args.seed is not None:
-        args.configuration.seed = args.seed
-        # numpy.random.seed((args.seed) % (2 ** 32 - 1))
-        random.seed((args.seed) % (2 ** 32 - 1))
-    else:
-        # numpy.random.seed(None)
-        random.seed(None)
-
-    if not os.path.exists(args.configuration.serialise.files.output_dir):
-        try:
-            os.makedirs(args.configuration.serialise.files.output_dir)
-        except (OSError, PermissionError) as exc:
-            logger.error("Failed to create the output directory!")
-            logger.exception(exc)
-            raise
-    elif not os.path.isdir(args.configuration.serialise.files.output_dir):
-        logger.error(
-            "The specified output directory %s exists and is not a file; aborting",
-            args.configuration.serialise.files.output_dir)
-        raise OSError("The specified output directory %s exists and is not a file; aborting" %
-                      args.configuration.serialise.files.output_dir)
-
-    if args.configuration.db_settings.dbtype == "sqlite":
-        args.configuration.db_settings.db = path_join(
-            args.configuration.serialise.files.output_dir,
-            args.configuration.db_settings.db)
-
-    if args.log is not None:
-        args.configuration.serialise.files.log = args.log
-
-    args.configuration.log_settings.log = args.configuration.serialise.files.log[:]
-
-    if args.configuration.serialise.files.log is not None and args.configuration.serialise.files.log != "":
-        if args.log != args.configuration.serialise.files.log and args.log is not None:
-            args.configuration.serialise.files.log = args.log
-        if not isinstance(args.configuration.serialise.files.log, str):
-            args.configuration.serialise.files.log.close()
-            args.configuration.serialise.files.log = args.configuration.serialise.files.log.name
-
-        log = args.configuration.serialise.files.log
-        if os.path.dirname(log) == "":
-            args.configuration.serialise.files.log = \
-                os.path.join(args.configuration.serialise.files.output_dir,
-                             os.path.basename(log))
-        else:
-            logdir = os.path.dirname(log).rstrip(os.path.sep)
-            logdir = os.path.relpath(logdir, args.configuration.serialise.files.output_dir)
-            args.configuration.serialise.files.log = \
-                os.path.join(args.configuration.serialise.files.output_dir,
-                             logdir,
-                             os.path.basename(log))
-        # path_join(args.configuration.serialise.files.output_dir, args.configuration.serialise.files.log)
-        handlers = logger.handlers[:]
-        for handler in handlers:
-            # if hasattr(handler, "baseFilename"):
-            logger.removeHandler(handler)
-
-        os.makedirs(os.path.dirname(args.configuration.serialise.files.log), exist_ok=True)
-        open(args.configuration.serialise.files.log, "wt").close()
-        handler = logging.FileHandler(args.configuration.serialise.files.log, mode="wt", delay=False)
-        handler.setFormatter(formatter)
-        logger.addHandler(handler)
+    mikado_configuration = _set_serialise_files_options(args=args, conf=mikado_configuration, logger=logger)
+    mikado_configuration = _set_serialise_run_options(mikado_configuration, args, logger=logger)
+    mikado_configuration = configurator.load_and_validate_config(mikado_configuration)
+    mikado_configuration, logger = check_log_settings_and_create_logger(mikado_configuration,
+                                                                        args.log, args.log_level,
+                                                                        section="serialise")
 
     logger.setLevel("INFO")
-    try:
-        logger.info("Command line: %s", " ".join(sys.argv))
-    except FileNotFoundError:
-        for handler in logger.handlers:
-            print("Handler:", handler.baseFilename)
-        raise
+    logger.info("Command line: %s", " ".join(sys.argv))
+    if args.random_seed is True:
+        mikado_configuration.seed = None
+    elif args.seed is not None:
+        mikado_configuration.seed = args.seed
+    else:
+        pass
 
-    logger.info("Random seed: %s", args.configuration.seed)
-    logger.setLevel(args.configuration.log_settings.log_level)
-
-    if args.configuration.serialise.files.junctions:
-        if args.configuration.reference.genome_fai in (None, ""):
-            if args.configuration.reference.genome not in (None, ""):
-                _ = pysam.Fastafile(args.configuration.reference.genome)
-                args.configuration.reference.genome_fai = args.configuration.reference.genome + ".fai"
-            else:
-                logger.critical("Missing FAI file for junction loading!")
-                sys.exit(1)
-
-    # File with the external scores
-    if args.external_scores is not None:
-        args.configuration.serialise.files.external_scores = args.external_scores
-
-    if args.codon_table not in (None, False, True):
-        args.configuration.serialise.codon_table = str(args.codon_table)
+    mikado_configuration.check()
+    random.seed(mikado_configuration.seed)
+    logger.info("Random seed: %s", mikado_configuration.seed)
+    logger.setLevel(mikado_configuration.log_settings.log_level)
 
     # Add sqlalchemy logging
     sql_logger = logging.getLogger("sqlalchemy.engine")
     if args.log_level == "DEBUG":
-        level = args.configuration.log_settings.log_level
+        level = mikado_configuration.log_settings.log_level
     else:
-        level = args.configuration.log_settings.sql_level
+        level = mikado_configuration.log_settings.sql_level
 
     sql_logger.setLevel(level)
     sql_logger.addHandler(logger.handlers[0])
 
-    logger.info("Using a %s database (location: %s)",
-                args.configuration.db_settings.dbtype,
-                args.configuration.db_settings.db)
+    logger.info("Using a %s database (location: %s)", mikado_configuration.db_settings.dbtype,
+                mikado_configuration.db_settings.db)
 
-    logger.info("Requested %d threads, forcing single thread: %s",
-                args.configuration.threads,
-                args.configuration.serialise.single_thread)
+    logger.info("Requested %d threads, forcing single thread: %s", mikado_configuration.threads,
+                mikado_configuration.serialise.single_thread)
 
-    if args.configuration.serialise.force is True:
-        if (args.configuration.db_settings.dbtype == "sqlite" and
-                os.path.exists(args.configuration.db_settings.db)):
-            logger.warn("Removing old data from %s because force option in place",
-                        args.configuration.db_settings.db)
-            os.remove(args.configuration.db_settings.db)
+    if mikado_configuration.serialise.force is True:
+        _execute_force_removal(mikado_configuration, logger=logger)
 
-        engine = dbutils.connect(args.configuration)
-        meta = sqlalchemy.MetaData(bind=engine)
-        meta.reflect(engine)
-        for tab in reversed(meta.sorted_tables):
-            logger.debug("Dropping %s", tab)
-            tab.drop()
-            if args.configuration.db_settings.dbtype == "mysql":
-                engine.execute("OPTIMIZE TABLE {}".format(tab.name))
-        if args.configuration.db_settings.dbtype == "mysql":
-            engine.execute("")
-        # This would fail in MySQL as it uses the OPTIMIZE TABLE syntax above
-        elif args.configuration.db_settings.dbtype != "sqlite":
-            engine.execute("VACUUM")
-        dbutils.DBBASE.metadata.create_all(engine)
-
-    return args, logger, sql_logger
+    return mikado_configuration, logger, sql_logger
 
 
 def serialise(args):
@@ -370,23 +333,13 @@ def serialise(args):
     :return:
     """
 
-    args, logger, sql_logger = setup(args)
-
-    # logger.info("Command line: %s",  " ".join(sys.argv))
-    load_orfs(args, logger)
-    load_blast(args, logger)
-    load_external(args, logger)
-    load_junctions(args, logger)
+    mikado_configuration, logger, sql_logger = setup(args)
+    load_orfs(mikado_configuration, logger)
+    load_blast(mikado_configuration, logger)
+    load_external(mikado_configuration, logger)
+    load_junctions(mikado_configuration, logger)
     logger.info("Finished")
-    try:
-        return 0
-    except KeyboardInterrupt:
-        raise
-    except Exception as exc:
-        logger.exception(exc)
-    finally:
-        logging.shutdown()
-        return 0
+    logging.shutdown()
 
 
 def serialise_parser():
@@ -461,6 +414,7 @@ a valid start codon.""")
 
     junctions = parser.add_argument_group()
     junctions.add_argument("--genome_fai", default=None)
+    junctions.add_argument("--genome", default=None)
     junctions.add_argument("--junctions", type=str, default=None)
 
     external_args = parser.add_argument_group()
@@ -474,11 +428,14 @@ a valid start codon.""")
     generic.add_argument("-mo", "--max-objects", dest="max_objects",
                          type=int, default=None,  # So it can actually be set through the JSON
                          help="""Maximum number of objects to cache in memory before
-                         committing to the database. Default: 100,000 i.e.
-                         approximately 450MB RAM usage for Drosophila.""")
-    generic.add_argument("-f", "--force", action="store_true", default=False,
+committing to the database. Default: 100,000 i.e. approximately 450MB RAM usage for Drosophila.""")
+    forcing = parser.add_mutually_exclusive_group()
+    forcing.add_argument("--no-force", action="store_false", default=None, dest="force",
+                         help="""Flag. If set, do not drop the contents of an existing Mikado DB
+before beginning the serialisation.""")
+    forcing.add_argument("--force", action="store_true", default=None, dest="force",
                          help="""Flag. If set, an existing databse will be deleted (sqlite)
-                         or dropped (MySQL/PostGreSQL) before beginning the serialisation.""")
+or dropped (MySQL/PostGreSQL) before beginning the serialisation.""")
     # If None, the default configuration will be used (from the blueprint)
     generic.add_argument("--configuration", "--json-conf", default=None,
                          dest="configuration", type=str,
@@ -493,10 +450,15 @@ a valid start codon.""")
                          help="Log level. Default: derived from the configuration; if absent, INFO")
     log_arguments.add_argument("--verbose", default=None, dest="log_level", action="store_const", const="DEBUG")
     log_arguments.add_argument("--quiet", default=None, dest="log_level", action="store_const", const="WARNING")
+    log_arguments.add_argument("--blast-loading-debug", default=None,
+                               dest="blast_loading_debug", action="store_true",
+                               help="Flag. If set, Mikado will switch on the debug mode for the XML/TSV loading.")
     generic.add_argument("db", type=str, default=None,
                          nargs='?',
                          help="Optional output database. Default: derived from configuration")
-    generic.add_argument("--seed", type=int, default=None,
-                         help="Random seed number.")
+    seed_group = parser.add_mutually_exclusive_group()
+    seed_group.add_argument("--seed", type=int, default=None, help="Random seed number. Default: 0.")
+    seed_group.add_argument("--random-seed", action="store_true", default=False,
+                            help="Generate a new random seed number (instead of the default of 0)")
     parser.set_defaults(func=serialise)
     return parser
