@@ -4,6 +4,7 @@
 This module defines a child of the Transcript class, which is used
 to verify that e.g. the assigned strand is correct.
 """
+import functools
 
 from .transcript import Transcript
 from ..exceptions import IncorrectStrandError, InvalidTranscript, InvalidCDS
@@ -118,16 +119,6 @@ class TranscriptChecker(Transcript):
 
         return Seq.reverse_complement(string)
 
-    @property
-    def is_reference(self):
-        return self.__is_reference
-
-    @is_reference.setter
-    def is_reference(self, value):
-        if value is not None and not isinstance(value, bool):
-            raise TypeError("This property only accepts boolean values")
-        self.__is_reference = value
-
     @property
     def strand_specific(self):
         """
@@ -182,18 +173,9 @@ class TranscriptChecker(Transcript):
         The finalize method is called preliminarly before any operation.
         """
 
-        try:
-            self.finalize()
-        except (InvalidTranscript, InvalidCDS) as exc:
-            raise InvalidTranscript("{} cannot be finalised correctly. Discarding it. Error: {}".format(self.id, exc))
+        self.finalize()
 
-        if self.exons[0][0] - self.start != 0:
-            error = "First exon start and transcript start disagree in {}: {} vs {}".format(self.id,
-                                                                                            self.exons[0][0],
-                                                                                            self.start)
-            self.logger.error(error)
-            raise InvalidTranscript(error)
-        if self.exons[-1][1] - self.start + 1 != len(self.fasta_seq):
+        if self.end - self.start + 1 != len(self.fasta_seq):
             error = """For {}, the expected length derived from last exon vs transcript start ({} vs {}) is different
             from the length of the FASTA sequence: {} vs {}""".format(self.id,
                                                                       self.exons[-1][1],
@@ -345,41 +327,33 @@ we will not reverse it")
 
         if self.is_coding is False:
             return
-        else:
-            try:
-                orfs = list(self.get_internal_orf_beds())
-            except AssertionError as exc:  # Invalid ORFs found
-                if self.strip_faulty_cds is False:
-                    raise InvalidTranscript("Invalid ORF(s) for {}. Discarding it. Error: {}".format(self.id, exc))
-                else:
-                    self.logger.warning("Invalid ORF(s) for %s. Stripping it of its CDS. Error: %s",
-                                        self.id, exc)
-                    self.strip_cds()
-                return
 
-            if len(orfs) > 1:
-                self.logger.warning("Multiple ORFs found for %s. Only considering the primary.", self.id)
-            elif len(orfs) == 0:
-                raise IndexError("No ORFs retrieved!")
+        try:
+            orfs = list(self.get_internal_orf_beds())
+            assert len(orfs) >= 1
+        except AssertionError as exc:  # Invalid ORFs found
+            if self.strip_faulty_cds is False:
+                raise InvalidTranscript("Invalid ORF(s) for {}. Discarding it. Error: {}".format(self.id, exc))
+            else:
+                self.logger.warning("Invalid ORF(s) for %s. Stripping it of its CDS. Error: %s",
+                                    self.id, exc)
+                self.strip_cds()
+            return
 
-            orfs = self.find_overlapping_cds(orfs)
+        orfs = self.find_overlapping_cds(orfs)
+        assert len(orfs) >= 1
 
-            if len(orfs) > 1:
-                self.logger.warning("Multiple ORFs found for %s. Only considering the primary.", self.id)
-            elif len(orfs) == 0:
-                raise IndexError("No ORFs retrieved!")
-
-            orf = orfs[0]
+        for orf_counter, orf in enumerate(orfs, start=1):
             assert isinstance(orf, BED12)
-
-            if orf.invalid:
+            if orf.invalid and orf_counter == len(orfs):
                 if self.strip_faulty_cds is False:
                     raise InvalidTranscript("Invalid ORF(s) for %s. Discarding it. Reason: %s", self.id,
                                             orf.invalid_reason)
                 else:
                     self.logger.warning("Invalid ORF for %s (reason: %s)", self.id, orf.invalid_reason)
                     self.strip_cds(self.strand_specific)
-            else:
+                return
+            elif not orf.invalid:
                 self.logger.debug("%s %s a valid start codon, %s a valid stop codon",
                                   self.id, "has" if orf.has_start_codon else "does not have",
                                   "has" if orf.has_stop_codon else "does not have")
@@ -387,36 +361,30 @@ we will not reverse it")
                 self.has_stop_codon = orf.has_stop_codon
                 self.attributes["has_start_codon"] = orf.has_start_codon
                 self.attributes["has_stop_codon"] = orf.has_stop_codon
-
-            return
+                return
 
     @property
     def cdna(self) -> str:
 
         """This property calculates the cDNA sequence of the transcript."""
 
-        sequence = ''
+        return self.calculate_cdna(str(self.fasta_seq.seq), tuple(self.exons), self.cdna_length, self.strand,
+                                   self.start)
 
-        for exon in self.exons:
-            start = exon[0] - self.start
-            end = exon[1] + 1 - self.start
-            _ = str(self.fasta_seq[start:end].seq)
+    @staticmethod
+    @functools.lru_cache(maxsize=10, typed=True)
+    def calculate_cdna(fasta_seq, exons, cdna_length, strand, tstart):
+        sequence = ""
+        for exon in exons:
+            start = exon[0] - tstart
+            end = exon[1] + 1 - tstart
+            _ = fasta_seq[start:end]
+            assert len(_) == end - start, (len(_), end - start)
             sequence += _
 
-        # pylint: disable=no-member
-        if not isinstance(sequence, str):
-            if hasattr(sequence, "seq"):
-                sequence = str(sequence.seq)
-            else:
-                raise TypeError("Invalid object for sequence: {0} ({1})".format(
-                    type(sequence),
-                    repr(sequence)
-                ))
-        # pylint: enable=no-member
-
-        assert len(sequence) == self.cdna_length
-        if self.strand == "-":
-            sequence = self.rev_complement(sequence)
+        assert len(sequence) == cdna_length, (len(sequence), cdna_length)
+        if strand == "-":
+            sequence = TranscriptChecker.rev_complement(sequence)
         return sequence
 
     @property
